@@ -14,7 +14,7 @@ use log::{debug, error, info, trace, warn};
 use sn_data_types::{Blob, PrivateBlob, PublicBlob, PublicKey, TransferValidated};
 use sn_messaging::{
     client::{BlobRead, ClientMsg, ClientSigned, Cmd, DataQuery, ProcessMsg, Query, QueryResponse},
-    section_info::Message as SectionInfoMsg,
+    section_info::SectionInfoMsg,
     MessageId,
 };
 use std::{collections::BTreeSet, net::SocketAddr, time::Duration};
@@ -45,14 +45,32 @@ impl Session {
 
         self.endpoint = Some(endpoint.clone());
         let mut bootstrap_nodes = endpoint
+            .clone()
             .bootstrap_nodes()
             .to_vec()
             .into_iter()
             .collect::<BTreeSet<_>>();
-        let connected_elders = self.connected_elders.clone();
+        // let connected_elders = self.connected_elders.clone();
+
+        let cloned_endpoint = endpoint.clone();
         let _ = tokio::spawn(async move {
             while let Some(disconnected_peer) = disconnections.next().await {
-                let _ = connected_elders.lock().await.remove(&disconnected_peer);
+                // we assume elders should have high connectivity.
+                // any problem there and they'd be voted off and we'd get an updated section
+                // so just keep trying to reconnect
+                warn!(
+                    "Disconnected from elder {:?}. Attempting to reconnect",
+                    disconnected_peer
+                );
+                match cloned_endpoint.connect_to(&disconnected_peer).await {
+                    Ok(_) => info!("Reconnected to {:?}", disconnected_peer),
+                    Err(error) => {
+                        warn!(
+                            "Could not reconnect to {:?}, error: {:?}",
+                            disconnected_peer, error
+                        );
+                    }
+                };
             }
         });
 
@@ -70,7 +88,7 @@ impl Session {
             )
             .await
             {
-                we_have_keyset = self.section_key_set.lock().await.is_some();
+                we_have_keyset = self.section_key_set.read().await.is_some();
             } else {
                 // Remove the unresponsive peer we boostrapped to and bootstrap again
                 let _ = bootstrap_nodes.remove(&bootstrapped_peer);
@@ -89,7 +107,7 @@ impl Session {
 
         debug!(
             "Successfully obtained the list of Elders to send all messages to: {:?}",
-            self.connected_elders.lock().await.keys()
+            self.connected_elders.read().await.keys()
         );
 
         Ok(())
@@ -100,7 +118,7 @@ impl Session {
         let msg_id = MessageId::new();
         let endpoint = self.endpoint()?.clone();
 
-        let elders: Vec<SocketAddr> = self.connected_elders.lock().await.keys().cloned().collect();
+        let elders: Vec<SocketAddr> = self.connected_elders.read().await.keys().cloned().collect();
         debug!(
             "Sending command w/id {:?}, to {} Elders",
             msg_id,
@@ -178,7 +196,7 @@ impl Session {
             cmd, msg_id
         );
         let endpoint = self.endpoint()?.clone();
-        let elders: Vec<SocketAddr> = self.connected_elders.lock().await.keys().cloned().collect();
+        let elders: Vec<SocketAddr> = self.connected_elders.read().await.keys().cloned().collect();
         let pending_transfers = self.pending_transfers.clone();
 
         let section_pk = self
@@ -195,7 +213,7 @@ impl Session {
         });
         let msg_bytes = msg.serialize(dest_section_name, section_pk)?;
 
-        let _ = pending_transfers.lock().await.insert(msg_id, sender);
+        let _ = pending_transfers.write().await.insert(msg_id, sender);
 
         // Send message to all Elders concurrently
         let mut tasks = Vec::default();
@@ -259,7 +277,7 @@ impl Session {
         // connected Elders to the data we are querying
         let elders: Vec<SocketAddr> = self
             .connected_elders
-            .lock()
+            .read()
             .await
             .clone()
             .into_iter()
@@ -285,7 +303,16 @@ impl Session {
         // We send the same message to all Elders concurrently
         let mut tasks = FuturesUnordered::new();
         let (sender, mut receiver) = channel::<QueryResponse>(7);
-        let _ = pending_queries.lock().await.insert(msg_id, sender);
+
+        let pending_queries_for_thread = pending_queries.clone();
+        let _ = tokio::spawn(async move {
+            // Remove the response sender
+            trace!("Inserting channel for {:?}", msg_id);
+            let _ = pending_queries_for_thread
+                .write()
+                .await
+                .insert(msg_id, sender);
+        });
 
         // Set up response listeners
         for socket in elders {
@@ -308,7 +335,7 @@ impl Session {
                         );
                         result = Err(Error::SendingQuery);
                     } else {
-                        trace!("ClientMsg with {:?} sent to {}", &msg_id, &socket);
+                        trace!("ClientMsg with id: {:?}, sent to {}", &msg_id, &socket);
                         result = Ok(());
                         break;
                     }
@@ -386,8 +413,8 @@ impl Session {
                 }
                 (Err(error), _) => {
                     error!(
-                        "Timeout while waiting for response to client request: {:?}",
-                        &error
+                        "Timeout while waiting for response to client request w/ id {:?}: {:?}",
+                        &msg_id, &error
                     );
                     break None;
                 }
@@ -402,9 +429,11 @@ impl Session {
             msg_id, response
         );
 
-        // Remove the response sender
-        trace!("Removing channel for {:?}", msg_id);
-        let _ = pending_queries.lock().await.remove(&msg_id);
+        let _ = tokio::spawn(async move {
+            // Remove the response sender
+            trace!("Removing channel for {:?}", msg_id);
+            let _ = pending_queries.clone().write().await.remove(&msg_id);
+        });
 
         response
             .map(|response| QueryResult { response, msg_id })
@@ -433,11 +462,8 @@ impl Session {
         // FIXME: we don't know our section PK. We must supply a pk for now we do a random one...
         let random_section_pk = threshold_crypto::SecretKey::random().public_key();
 
-        let msg = SectionInfoMsg::GetSectionQuery {
-            public_key: client_pk,
-            is_node: false,
-        }
-        .serialize(dest_section_name, random_section_pk)?;
+        let msg = SectionInfoMsg::GetSectionQuery(client_pk)
+            .serialize(dest_section_name, random_section_pk)?;
 
         self.endpoint()?
             .send_message(msg, bootstrapped_peer)
@@ -466,12 +492,12 @@ impl Session {
             warn!("Not attempted to connect, insufficient elders yet known");
         }
 
-        let new_elders = self.all_known_elders.lock().await.clone();
+        let new_elders = self.all_known_elders.read().await.clone();
         let peers_len = new_elders.len();
 
         trace!("We now know our {} Elders.", peers_len);
         {
-            let mut session_elders = self.connected_elders.lock().await;
+            let mut session_elders = self.connected_elders.write().await;
             *session_elders = new_elders;
         }
 
